﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml.Serialization;
using System.Web;

namespace KnockoutServerSideViewModel.Web
{
    public class OrderDetail
    {
        public string ProductCode { get; set; }
        public int Quantity { get; set; } //Can be an int as it is managed by a drop down list
        public string UnitPrice { get; set; } //It's string so I can validate it server side
        public bool PriceNotValid { get; set; }
        public string Total
        {
            get
            {
                var result = "";
                if(!PriceNotValid)
                {
                    result = (decimal.Parse(UnitPrice) * Quantity).ToString();
                }
                return result;
            }
        }
        public bool Delivered { get; set; }
        public string Name { get; set; }
        public bool Editing { get; set; }
        public string Key { get; set; }
    }

    public class Page
    {
        public int Index { get; set; }
        public string Description { get; set; }
    }

    public class OrderViewModel : BaseViewModel
    {

        private static string PATH = HttpContext.Current.Server.MapPath("Order.xml");


        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string LastSavedTime { get; set; }
        public int TimesSaved { get; set; }
        public int FirstElement { get; set; }
        public int CurrentPage { get; set; }
        public int PageSize { get; set; }
        public int TotalSize { get; set; }
        public string Notes { get; set; }
        public bool MainEditing { get; set; }
        //public string Total
        //{
        //    get
        //    {
        //        return Details.Sum(p => p.UnitPrice * p.Quantity).ToString();
        //    }
        //}

        public List<OrderDetail> Details { get; set; }
        public List<Page> Pages { get; set; }

        public void SetPaging()
        {

            var prevPageLink = false;
            var nextPageLink = false;
            int maxPage = (this.TotalSize - 1) / this.PageSize + 1;
            int firstPage = (this.CurrentPage - 1) - ((this.CurrentPage - 1) % 10) + 1;
            if (firstPage > 1)
            {
                prevPageLink = true;
            }
            int lastPage = firstPage + 9;
            if (lastPage > maxPage)
            {
                lastPage = maxPage;
            }
            if (TotalSize / PageSize > lastPage)
            {
                nextPageLink = true;
            }
            List<Page> pages = new List<Page>();
            if (prevPageLink)
            {
                pages.Add(new Page()
                {
                    Description = "<<<",
                    Index = firstPage - 1
                });
            }
            for (int i = firstPage; i <= lastPage; i++)
            {
                pages.Add(new Page()
                {
                    Description = i.ToString(),
                    Index = i
                });
            }
            if (nextPageLink)
            {
                pages.Add(new Page()
                {
                    Description = ">>>",
                    Index = lastPage + 1
                });
            }
            this.Pages = pages; ;

        }

        public void Save()
        {
            this.TimesSaved++;
            this.LastSavedTime = DateTime.Now.ToLongDateString() + " " + DateTime.Now.ToLongTimeString();
            PersistModel();
        }

        public void SaveAndClose()
        {
            this.TimesSaved++;
            this.LastSavedTime = DateTime.Now.ToLongDateString() + " " + DateTime.Now.ToLongTimeString();
            PersistModel();
            this.RedirectUrl="/Thanks.aspx";
        }

        public void SetDelivered()
        {
            this.Details.Where(p => p.Key.ToString() == Argument).First().Delivered = true;
            PersistModel();
        }

        public void SetNotDelivered()
        {
            this.Details.Where(p => p.Key.ToString() == Argument).First().Delivered = false;
            PersistModel();
        }

        public void Edit()
        {
            this.Details.Where(p => p.Key.ToString() == Argument).First().Editing = true;
            
        }

        public void Close()
        {
            decimal price;
            if (decimal.TryParse(this.Details.Where(p => p.Key.ToString() == Argument).Single().UnitPrice, out price))
            {
                this.Details.Where(p => p.Key.ToString() == Argument).First().Editing = false;
                this.Details.Where(p => p.Key.ToString() == Argument).First().PriceNotValid = false;
            }
            else
            {
                this.Details.Where(p => p.Key.ToString() == Argument).First().PriceNotValid = true;
            }
            
        }
        
        public void Delete()
        {
            var order  = GetOrder();
            order.Details.Remove(order.Details.Where(p => p.Key.ToString() == Argument).First());
            SaveModelToXml(order);
            order = GetPagedOrder(this.CurrentPage, this.PageSize);
            this.Details = order.Details;
        }

      public void ChangePage()
        {
            int pageIndex = int.Parse(this.Argument);
            var order = GetPagedOrder(pageIndex, PageSize);
            this.Details = order.Details;
            this.CurrentPage = pageIndex;
            SetPaging();
        }


      public void MainEdit()
      {
          this.MainEditing = true;
      }

      public void MainClose()
      {
          this.MainEditing = false;
          Save();
      }
        public void AddTwoEuro()
        {
            var detail = this.Details.Where(p => p.Key.ToString() == Argument).First();
            detail.UnitPrice = (decimal.Parse(detail.UnitPrice) + 2).ToString();
            Save();
        }

        public void AddThreeEuro()
        {
            var detail = this.Details.Where(p => p.Key.ToString() == Argument).First();
            detail.UnitPrice += 3;
        }

        public void PersistModel()
        {
            var order = GetOrder();
            order.FirstName = this.FirstName;
            order.LastName = this.LastName;

            foreach (var detail in this.Details)
            {
                var oldDetail = order.Details.Where(p => p.Key == detail.Key).SingleOrDefault();
                if (oldDetail != null)
                {
                    oldDetail.Delivered = detail.Delivered;
                    oldDetail.Editing = false; ;
                    oldDetail.Name = detail.Name;
                    oldDetail.Quantity = detail.Quantity;
                    oldDetail.UnitPrice = detail.UnitPrice;
                    oldDetail.ProductCode = detail.ProductCode;
                }
                else
                {
                    order.Details.Add(detail);
                }
            }

            //SaveModelToXml(order);

            this.Details = order.Details;
            this.TotalSize = this.Details.Count();
            FirstElement = (CurrentPage - 1) * PageSize + 1;
            this.Details = this.Details.Skip(FirstElement - 1).Take(PageSize).ToList();
        }

        private static void SaveModelToXml(OrderViewModel order)
        {
            System.Xml.Serialization.XmlSerializer x = new System.Xml.Serialization.XmlSerializer(order.GetType());
            TextWriter tw = new StreamWriter(PATH);
            x.Serialize(tw, order);
            tw.Close();
        }

        public static OrderViewModel GetPagedOrder(int currentPage, int pageSize)
        {
            OrderViewModel b = GetOrder();
            b.FirstElement = (currentPage - 1) * pageSize + 1;
            b.PageSize = pageSize;
            b.TotalSize = b.Details.Count();
            b.Details = b.Details.Skip(b.FirstElement - 1).Take(pageSize).ToList();
            b.CurrentPage = currentPage;
            b.SetPaging();
            return b;
        }

        private static OrderViewModel GetOrder()
        {
            XmlSerializer serializer = new XmlSerializer(typeof(OrderViewModel));
            TextReader tr = new StreamReader(PATH);
            OrderViewModel b = (OrderViewModel)serializer.Deserialize(tr);
            tr.Close();
            b.LastSavedTime = DateTime.Now.ToLongDateString() + " " + DateTime.Now.ToLongTimeString();
            return b;
        }

    }

    
}
